from Products.CMFPlone.browser.navtree import buildFolderTree, DefaultNavtreeStrategy, NavtreeQueryBuilder
from Products.CMFCore.utils import getToolByName


class _CustomStrategy(DefaultNavtreeStrategy):
    """A navtree strategy which makes sure we descend all the way to the currentItem, even if the `depth` restriction wouldn't take us down that far."""
    
    def __init__(self, context, rootPath, view=None):
        """`rootPath` should be absolute, not portal-relative."""
        DefaultNavtreeStrategy.__init__(self, context, view)
        self.rootPath = rootPath


def customNavtree(self, basePath='.', types=None, currentItem=None, filter=None):
    """Return a navtree suitable for passing to portlet_navtree_macro().
    
    The tree is rooted at `basePath`, limited to the types given by `types`, and
    has `currentItem` highlighted. This is much easier than calling
    buildFolderTree directly.
    
    Parameters:
        basePath --
            The path of the subtree to descend. Should be relative to the portal
            root, start with a /, and not end with a /. Omit to use the current
            context as the base path. Use an empty string to indicate the portal
            root.
        types --
            A list of type names, like ['Folder', 'Page']. Only items whose
            types are in the list will be returned. Omit to default to site-wide
            settings.
        currentItem --
            Where the user currently is. The tree will be expanded to show the
            ancestors of `currentItem`, the children of `currentItem`, and the
            children of the ancestors of `currentItem`.
            * Pass a portal-dwelling object as currentItem, and a 'currentItem'
              key will be set to True in the object's corresponding result dict.
              'currentParent' will be set to True for the dicts of its
              ancestors. (If `currentItem` is not within `basePath`, no
              'currentItem' or 'currentParent' keys will be set to True.)
            * To default to `context` (that is, whatever the user is presently
              looking at), omit altogether.
        filter --
            A catalog query to further refine the results
    
    Returns a tree, represented as a dict with these keys:
        item            -   A catalog brain of this item
        depth           -   The depth of this item, relative to the `basePath`
                            level. (Direct children of `basePath` have a depth
                            of 1.)
        currentItem     -   True if this is the current item
        currentParent   -   True if this is a direct parent of the current item
        children        -   A list of children nodes (represented as a list of
        	                dicts like this dict) of this node
    
    Example:
        tree = context.customNavtree(basePath='/someFolder')
        return context.portlet_navtree_macro(children=tree.get('children', []),
            show_children=True, isNaviTree=True)
    
    """
    # Thanks to Martin Aspeli for his original getSections script. This was, once upon a time, supposed to be a facade around buildFolderTree that exposed the same interface as getSections, but it's since moved on. Still, some of the code remains based on it.
    
    def nearestFolderAbove(node):
        """If `node` is folderish, return `node`. Else, return the folderish item containing `node`."""
        return node.is_folderish() and node or node.aq_inner.aq_parent  # Does this even work? (Apparently.) How about node.getParentNode() instead?
    
    def isBeneath(parentPathComponents, descendantPathComponents):
        """Return whether `descendantPathComponents` represents a subtree of (or is identical to) `parentPathComponents`."""
        return tuple(parentPathComponents) == tuple(descendantPathComponents[:len(parentPathComponents)])  # tuple casts in case the operands aren't the same type; equivalent tuples and lists aren't equal
    
    context = self
    portalPath = getToolByName(context, 'portal_url').getPortalPath()
    
    # Normalize args:
    if not currentItem:  # If no explicit currentItem is passed in, default to whatever the user is looking at.
        currentItem = context
    if basePath == '.':
        absoluteBasePath = '/'.join(nearestFolderAbove(context).getPhysicalPath())
    else:
        absoluteBasePath = portalPath + basePath
    if filter is None:  # Something writes to filter if I leave it a mutable default arg.
        filter = {}
    
    # Start from the defaults the stock navtree uses.
    query = NavtreeQueryBuilder(context)()
    
    # And then pave over the parts we have differing opinions on:
    query.update(filter)
    # We normally put the path of currentItem in the query (or the nearest folder above it, if it's not folderish), so we get its children, which we want to show. If currentItem isn't within absoluteBasePath, however, query for absoluteBasePath instead; otherwise, by querying for currentItem, we won't find the contents of absoluteBasePath at all, and the rootPath restriction applied later by our _CustomStrategy will filter everything out, leaving the tree empty.
    pathToQuery = isBeneath(absoluteBasePath.split('/'), currentItem.getPhysicalPath()) and '/'.join(nearestFolderAbove(currentItem).getPhysicalPath()) or absoluteBasePath
    depthOfBasePathBelowPortal = len(absoluteBasePath.split('/')) - len(portalPath.split('/'))  # 1 if portal root is /portal and absoluteBasePath is /portal/someFolder
    query['path'].update({'query': pathToQuery, 'navtree_start': depthOfBasePathBelowPortal + 1})
    if types:
        query['portal_type'] = types

    return buildFolderTree(context, obj=currentItem, query=query, strategy=_CustomStrategy(context, absoluteBasePath))
